/***************************************************************************
 * Copyright (C) 2003-2004 Adam Geitgey <adam@rootnode.org>                *
 * Copyright (C) 2003 Hans Karlsson <karlsson.h@home.se>                   * 
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "misc_python.h"
#include <qobject.h>
#include "karamba.h"
#include "meter.h"
#include "meter_python.h"

/* now a method we need to expose to Python */
long acceptDrops(long widget) {
  karamba* currTheme = (karamba*)widget;

  currTheme->setAcceptDrops(TRUE);

  return 1;
}

PyObject* py_accept_drops(PyObject *self, PyObject *args)
{
  long widget;

  if (!PyArg_ParseTuple(args, "l", &widget))
    return NULL;
  return Py_BuildValue("l", acceptDrops(widget));
}

// Runs a command, returns 0 if it could not start command
PyObject* py_execute_command(PyObject* self, PyObject* args)
{
  char* command;
  if (!PyArg_ParseTuple(args, "s:execute", &command))
      return NULL;
  return Py_BuildValue("l", KRun::runCommand(command));
}

// Runs a command, returns 0 if it could not start command
PyObject* py_execute_command_interactive(PyObject* self, PyObject* args)
{
  long widget;
  //if (!PyArg_ParseTuple(args, "ls", &widget, &command))
  //  return NULL;

  int numLines;       /* how many lines we passed for parsing */
  char * line;        /* pointer to the line as a string */
  
  PyObject * listObj; /* the list of strings */
  PyObject * strObj;  /* one string in the list */
  
  /* the O! parses for a Python object (listObj) checked
     to be of type PyList_Type */
  if (! PyArg_ParseTuple( args, "lO!", &widget, &PyList_Type, &listObj)) 
    return NULL;
  

  karamba* currTheme = (karamba*)widget;
  
  currTheme->currProcess = new KProcess;
  


  /* get the number of lines passed to us */
  numLines = PyList_Size(listObj);
  
  /* should raise an error here. */
  if (numLines < 0) return NULL; /* Not a list */

  /* iterate over items of the list, grabbing strings, and parsing
     for numbers */
  for (int i=0; i<numLines; i++){
    
    /* grab the string object from the next element of the list */
    strObj = PyList_GetItem(listObj, i); /* Can't fail */
    
    /* make it a string */
    line = PyString_AsString( strObj );
    
    /* now do the parsing */
    *(currTheme->currProcess) << line;
   
  }
  QApplication::connect(currTheme->currProcess, SIGNAL(processExited(KProcess *)),
                        currTheme, SLOT(processExited(KProcess *)));
  QApplication::connect(currTheme->currProcess, SIGNAL(receivedStdout(KProcess *,
                        char *, int)),
                        currTheme, SLOT(receivedStdout(KProcess *, char *, int)));
  currTheme->currProcess->start(KProcess::NotifyOnExit, KProcess::Stdout);

  return Py_BuildValue("l", (int)(currTheme->currProcess->pid()));
}

long attachClickArea(long widget, long meter, QString LeftButton, QString MiddleButton, QString RightButton)
{
  karamba* currTheme = (karamba*) widget;
  Meter* currMeter = (Meter*) meter;

  // Look if currMeter has an ClickArea attached.
  bool meterAlreadyClickable = currTheme -> clickList -> containsRef(currMeter);

  // if currMeter is of type ImageLabel*
  if (ImageLabel* image = dynamic_cast<ImageLabel*>(currMeter))
  {
      image -> attachClickArea(LeftButton, MiddleButton, RightButton);
      if (!meterAlreadyClickable)
      {
          //qWarning("attachClickArea : meter is image");
          currTheme -> clickList -> append(image);
      }
  }
  // else if currMeter is of type TextLabel*
  else if (TextLabel* text = dynamic_cast<TextLabel*>(currMeter))
  {
      text -> attachClickArea(LeftButton, MiddleButton, RightButton);
      if (!meterAlreadyClickable)
      {
          //qWarning("attachClickArea : meter is text");
          currTheme -> clickList -> append(text);
      }
  }
  else
  {
      //The given meter does not support attached clickAreas.
      qWarning("The given meter is not of type image or text");
      return 0;
  }
  return 1;
}

PyObject* py_attach_clickArea(PyObject* self, PyObject* args, PyObject* dict)
{
  long widget;
  long meter;
  char* LeftButton = NULL;
  char* MiddleButton = NULL;
  char* RightButton = NULL;
  char* mouseButtons[] = {"Widget", "Meter", "LeftButton", "MiddleButton",
                          "RightButton", NULL};
  if (!PyArg_ParseTupleAndKeywords(args, dict, "ll|sss:attachClickArea",
   mouseButtons, &widget, &meter, &LeftButton, &MiddleButton, &RightButton))
    return NULL;
  QString lB, mB, rB;
  if (LeftButton != NULL)
  {
      lB.setAscii(LeftButton);
  }
  else
  {
      lB.setAscii("");
  }
  if (MiddleButton != NULL)
  {
      mB.setAscii(MiddleButton);
  }
  else
  {
      mB.setAscii("");
  }
  if (RightButton != NULL)
  {
       rB.setAscii(RightButton);
  }
  else
  {
       rB.setAscii("");
  }
  return Py_BuildValue("l", attachClickArea(widget, meter, lB, mB, rB));
}

/* now a method we need to expose to Python */
long toggleShowDesktop(long widget) {
  karamba* currTheme = (karamba*)widget;

  ShowDesktop *s = ShowDesktop::the();

  s->toggle();

  return 1;
}

PyObject* py_toggle_show_desktop(PyObject *self, PyObject *args)
{
  long widget, image, deg;
  if (!PyArg_ParseTuple(args, "l:toggleShowDesktop", &widget))
    return NULL;
  return Py_BuildValue("l", toggleShowDesktop(widget));
}

/* now a method we need to expose to Python */
const char* getThemePath(long widget) {
  karamba* currTheme = (karamba*)widget;

  return currTheme->themePath.ascii();
}

PyObject* py_get_theme_path(PyObject *self, PyObject *args)
{
  long widget;
  if (!PyArg_ParseTuple(args, "l:getThemePath", &widget))
    return NULL;
  return Py_BuildValue("s", getThemePath(widget));
}

/* now a method we need to expose to Python */
long createClickArea(long widget, long x, long y, long w, long h, char* text) {

  karamba* currTheme = (karamba*)widget;
  ClickArea *tmp = new ClickArea( x, y, w, h );
  QString onclick;

  onclick.setAscii(text);

  tmp->setOnClick(onclick );

  currTheme -> clickList -> append(tmp);
  return (int)tmp;
}

PyObject* py_create_click_area(PyObject *self, PyObject *args)
{
  long widget, x, y, w, h;
  char *text;
  if (!PyArg_ParseTuple(args, "llllls:createClickArea", &widget, &x, &y, 
                        &w, &h, &text))
    return NULL;
  return Py_BuildValue("l", createClickArea(widget, x, y, w, h, text));
}

/* now a method we need to expose to Python */
long openTheme(char* path) {

  QString filename;
  karamba* currTheme = 0; 

  filename.setAscii(path);
  
  QFileInfo file( filename );
  
  if( file.exists() )
    {
      currTheme = new karamba( filename, false );
      currTheme->show();
    }
  
  return (int)currTheme;
}

PyObject* py_open_theme(PyObject *self, PyObject *args)
{
  char *themePath;
  if (!PyArg_ParseTuple(args, "s:openTheme", &themePath))
    return NULL;
  return Py_BuildValue("l", openTheme(themePath));
}

