/***************************************************************************
 * Copyright (C) 2003-2004 Adam Geitgey <adam@rootnode.org>                *
 * Copyright (C) 2003 Hans Karlsson <karlsson.h@home.se>                   * 
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "misc_python.h"
#include <qobject.h>
#include "karamba.h"
#include "meter.h"
#include "meter_python.h"

long moveSystray(long widget, long x, long y, long w, long h) {
  karamba* currTheme = (karamba*)widget;
  
  if (currTheme->systray != 0) {
    currTheme->systray->move((int)x,(int)y);
    currTheme->systray->setMinimumSize((int)w,(int)h);
    currTheme->systray->show();
  }
  
  return 1;
}

PyObject* py_move_systray(PyObject *self, PyObject *args)
{
  long widget, x, y, w, h;
  if (!PyArg_ParseTuple(args, "lllll:moveSystray", &widget, &x, &y, &w, &h))
    return NULL;
  return Py_BuildValue("l", moveSystray(widget, x, y, w, h));
}

/* now a method we need to expose to Python */
long showSystray(long widget) {
  karamba* currTheme = (karamba*)widget;

  if (currTheme->systray != 0) {
    currTheme->systray->show();
  }
  
  return 1;
}

PyObject* py_show_systray(PyObject *self, PyObject *args)
{
  long widget;
  if (!PyArg_ParseTuple(args, "l:showSystray", &widget))
    return NULL;
  return Py_BuildValue("l", showSystray(widget));
}

/* now a method we need to expose to Python */
long hideSystray(long widget) {
  karamba* currTheme = (karamba*)widget;

  if (currTheme->systray != 0) {
    currTheme->systray->hide();
  }
  
  return 1;
}

PyObject* py_hide_systray(PyObject *self, PyObject *args)
{
  long widget;
  if (!PyArg_ParseTuple(args, "l:hideSystray", &widget))
    return NULL;
  return Py_BuildValue("l", hideSystray(widget));
}

/* now a method we need to expose to Python */
long createSystray(long widget, long x, long y, long w, long h) {
  karamba* currTheme = (karamba*)widget;

  //Don't create more than one systray
  if (currTheme->systray == 0) {
    currTheme->systray = new Systemtray(currTheme);    
    currTheme->systray->move((int)x,(int)y);
    currTheme->systray->setMinimumSize((int)w,(int)h);
    currTheme->systray->initSystray();
    currTheme->systray->show();
  }
  
  return 1;
}

PyObject* py_create_systray(PyObject *self, PyObject *args)
{
  long widget, x, y, w, h;
  if (!PyArg_ParseTuple(args, "lllll:createSystray", &widget, &x, &y, &w, &h))
    return NULL;
  return Py_BuildValue("l", createSystray(widget, x, y, w, h));
}
